import React from 'react';
import { findDOMNode } from 'react-dom';
import {
	Transition,
	CSSTransition,
	TransitionGroup,
} from 'react-transition-group'
import classnames from 'classnames';
import ListFiltersAddForm from './ListFiltersAddForm';
import Popout from '../../../shared/Popout';
import PopoutList from '../../../shared/Popout/PopoutList';
import PropTypes from 'prop-types'

import {
	Input
} from 'antdlib';
import { setFilter } from '../active/actions';

var createReactClass = require('create-react-class');
var ListFiltersAdd = createReactClass({
	displayName: 'ListFiltersAdd',
	propTypes: {
		maxHeight: PropTypes.number,
	},
	getDefaultProps () {
		return {
			maxHeight: 360,
		};
	},
	getInitialState () {
		return {
			innerHeight: 0,
			isOpen: false,
			searchString: '',
			selectedField: false,
		};
	},
	updateSearch (e) {
		this.setState({ searchString: e.target.value });
	},
	openPopout () {
		this.setState({ isOpen: true }, this.focusSearch);
	},
	closePopout () {
		this.setState({
			// innerHeight: 0,
			isOpen: false,
			searchString: '',
			selectedField: false,
		});
	},
	setPopoutHeight (height) {
		this.setState({ innerHeight: Math.min(this.props.maxHeight, height) });
	},
	navigateBack () {
		this.setState({
			selectedField: false,
			searchString: '',
			// innerHeight: ,
		}, this.focusSearch);
	},
	focusSearch () {
		findDOMNode(this.refs.search).focus();
	},
	selectField (field) {
		this.setState({
			selectedField: field,
		});
	},
	applyFilter (value) {
		this.props.dispatch(setFilter(this.state.selectedField.path, value));
		this.closePopout();
	},
	renderList () {
		const activeFilterFields = this.props.activeFilters.map(obj => obj.field);
		const activeFilterPaths = activeFilterFields.map(obj => obj.path);
		const { searchString } = this.state;
		let filteredFilters = this.props.availableFilters;

		if (searchString) {
			filteredFilters = filteredFilters
				.filter(filter => filter.type !== 'heading')
				.filter(filter => new RegExp(searchString)
				.test(filter.field.label.toLowerCase()));
		}

		const popoutList = filteredFilters.map((el, i) => {
			if (el.type === 'heading') {
				return (
					<PopoutList.Heading key={'heading_' + i}>
						{el.content}
					</PopoutList.Heading>
				);
			}

			const filterIsActive = activeFilterPaths.length && (activeFilterPaths.indexOf(el.field.path) > -1);

			return (
				<PopoutList.Item
					key={'item_' + el.field.path}
					icon={filterIsActive ? 'check' : 'right'}
					iconHover={filterIsActive ? 'check' : 'right'}
					isSelected={!!filterIsActive}
					label={el.field.label}
					onClick={() => { this.selectField(el.field); }} />
			);
		});

		const formFieldStyles = {
			borderBottom: '1px dashed rgba(0, 0, 0, 0.1)',
			marginBottom: '1em',
			paddingBottom: '1em',
		};

		return (
			<Popout.Pane onLayout={this.setPopoutHeight} key="list">
				<Popout.Body>
					<div style={formFieldStyles}>
						<Input
							onChange={this.updateSearch}
							placeholder="搜索过滤项"
							ref="search"
							value={this.state.searchString}
						/>
					</div>
					{popoutList}
				</Popout.Body>
			</Popout.Pane>
		);
	},
	renderForm () {
		return (
			<Popout.Pane onLayout={this.setPopoutHeight} key="form" className="">
				<ListFiltersAddForm
					activeFilters={this.props.activeFilters}
					field={this.state.selectedField}
					onApply={this.applyFilter}
					onCancel={this.closePopout}
					onBack={this.navigateBack}
					maxHeight={this.props.maxHeight}
					onHeightChange={this.setPopoutHeight}
					dispatch={this.props.dispatch}
				/>
			</Popout.Pane>
		);
	},
	render () {
		const { isOpen, selectedField } = this.state;
		const popoutBodyStyle = this.state.innerHeight
			? { height: this.state.innerHeight }
			: null;
		const popoutPanesClassname = classnames('Popout__pane', 'Popout__panes', {
			'Popout__scrollable-area': !selectedField,
		});
		const heightTransitionStyle = {
			width:'300px',
			overflow:'hidden',
			transition: `height 360ms`,
			transitionTimingFunction:'cubic-bezier(.36, .66, .04, 1)',
			height : this.state.innerHeight
		  }

		return (
			
			<div style={{
				
				// overflowX:'hidden',
				
			}}>
				
					<Popout.Header
							leftAction={selectedField ? this.navigateBack : null}
							leftIcon={selectedField ? 'left' : null}
							title={selectedField ? selectedField.label : '请选择过滤项'}
							transitionDirection={selectedField ? 'next' : 'prev'} />
					<div style={{
						position:'relative',
					}}>
						<TransitionGroup>
							
							<Transition>
								{(state) => {
									return <div  style={heightTransitionStyle}></div>
								}}
							</Transition>
							<CSSTransition
								key={selectedField ? 'Popout__pane-next' : 'Popout__pane-prev'}
								className={popoutPanesClassname}
								// style={popoutBodyStyle}
								// timeout={300}
								// classNames={'Popout__pane-next'}
								classNames={selectedField ? 'Popout__pane-next' : 'Popout__pane-prev'}
								mountOnEnter
								unmountOnExit
								addEndListener={(node, done) => {
									// use the css transitionend event to mark the finish of a transition
									node.addEventListener('transitionend', done, false);
								  }}
								>
									
										{selectedField ? this.renderForm() : this.renderList()}

							</CSSTransition>
									
						</TransitionGroup>

					</div>
					
			</div>
		
		);
	},
});

module.exports = ListFiltersAdd;
