import Field from '../Field';
import React from 'react';
import _ from 'lodash';
import URL from 'url';

/**
 * TODO:
 * - Custom path support
 */
import {
	Select,
	Input,
	Row,
	Col,
	Switch,
} from "antdlib";
import opts, {
	TYPE_TEXT,
	TYPE_BOOLEAN
} from './config'
const Option = Select.Option;

module.exports = Field.create({

	displayName: 'TypeValueField',
	statics: {
		type: 'TypeValue',
	},
	getInitialState () {
		return {
		};
	},
	shouldUseFieldDecorator(){
		return false;
	},
	format(){
		var value = this.props.form.getFieldValue(this.getInputName(this.props.path));
		value.type = value.type || 0;
		value.value = value.value == undefined ? '' : value.value;
		return JSON.stringify(value);
	},
	typeValueChanged (newValue) {
		const name = this.getInputName(this.props.path);
		let content = this.props.form.getFieldValue(name);

		let contentValue;

		switch(newValue){
			case TYPE_TEXT:
				contentValue = content.value + ""; // to string
			break;
			case TYPE_BOOLEAN:
				contentValue = content.value ? true : false;
			break;
		}
		
		this.props.form.setFieldsValue({
			[name]:_.assign(content, {
				type:newValue,
				value:contentValue,
			}),
		})
	},

	handleTextChange(e){
		const name = this.getInputName(this.props.path);
		let content = this.props.form.getFieldValue(name);
		let value = e.currentTarget.value
		this.props.form.setFieldsValue({
			[name]:_.assign(content, {value}),
		})
	},
	handleSwitchChanged(value){
		const name = this.getInputName(this.props.path);
		let content = this.props.form.getFieldValue(name);
		this.props.form.setFieldsValue({
			[name]:_.assign(content, {value}),
		})
	},
	renderValue () {
		var selected = this.props.ops.find(option => option.value === this.props.value);
		return <Input disabled>{selected ? selected.label : null}</Input>;
	},
	renderOptions(){
		return _.map(opts, (o) => {
			return <Select.Option key={o.value} value={o.value}>{o.label}</Select.Option>;
		})
	},

	renderField () {
		// TODO: This should be natively handled by the Select component
		// var ops = (this.props.numeric) ? this.props.ops.map(function (i) { return { label: i.label, value: i.value}; }) : this.props.ops;
		

		const name = this.getInputName(this.props.path);

		const { getFieldDecorator } = this.props.form;
		const hiddenInput = (


			getFieldDecorator(
				name,
				{
					rules: this.getValidateRules(),
					initialValue:this.getInitValue(),
				}
			)(
				<Input disabled type="hidden" name={name}  />
			)
		);


		let content = this.props.form.getFieldValue(name);

		let secondNode;
		
		switch(content.type){
			case TYPE_TEXT:
				secondNode = <Input
					ref='input'
					size={this.fieldSize}
					onChange={this.handleTextChange.bind(this)}
					defaultValue={content.value}
				/>
			break;
			case TYPE_BOOLEAN:
				secondNode = <Switch
								name={name}
								defaultChecked = {content.value}
								onChange={this.handleSwitchChanged.bind(this)}
								size="large"
							/>
			break;
		}

		const { autoFocus, inputProps, path } = this.props;
		return (
			<div>
				{hiddenInput}
				<Row gutter="15">
					<Col span="6">
						<Select {...{
							autoFocus,
							size:'large',
							onChange: this.typeValueChanged.bind(this),
							defaultValue:content.type,
						}}>
							{this.renderOptions()}
						</Select>
					</Col>
					<Col span="18">
						{secondNode}
					</Col>
				</Row>
				
					
				
				
			</div>
			
			
		);
	},

});
