var _ = require('lodash');
var FieldType = require('../Type');
var util = require('util');
var utils = require('keystone-utils');

/**
 * Select FieldType Constructor
 * @extends Field
 * @api public
 */
function goodpropertymarix (list, path, options) {
	this.ui = options.ui || 'GoodPropertyMarix';
	options.default = null;
	
	goodpropertymarix.super_.call(this, list, path, options);
}
goodpropertymarix.properName = 'GoodPropertyMarix';
util.inherits(goodpropertymarix, FieldType);

/**
 * Registers the field on the List's Mongoose Schema.
 *
 * Adds a virtual for accessing the label of the selected value,
 * and statics to the Schema for converting a value to a label,
 * and retrieving all of the defined options.
 */
goodpropertymarix.prototype.addToSchema = function (schema) {
	var field = this;
	var ops = (this._nativeType) ? _.defaults({ type: this._nativeType }, this.options) : this.options;
	schema.path(this.path, Object);
	this.bindUnderscoreMethods();
};


/**
 * Asynchronously confirms that the provided value is valid
 */
goodpropertymarix.prototype.validateInput = function (data, callback) {
	var value = this.getInputFromData(data);
	
	var result = value === undefined || value === null || value === '' || typeof value === 'object' ? true : false;
	utils.defer(callback, result);
};

/**
 * Asynchronously confirms that the provided value is present
 */
goodpropertymarix.prototype.validateRequiredInput = function (item, data, callback) {
	var value = this.getInputFromData(data);
	var result = false;
	if (value === undefined) {
		if (item.get(this.path)) {
			result = true;
		}
	} else if (value) {
		if (typeof value === 'object' && value.hasOwnProperty('propertyMap') && value.hasOwnProperty('marixValue')) {
			result = true;

		}
	}
	utils.defer(callback, result);
};

/**
 * Validates that a valid option has been provided in a data object
 *
 * Deprecated
 */
goodpropertymarix.prototype.inputIsValid = function (data, required, item) {
	if (data[this.path]) {
		return (data[this.path] in this.map) ? true : false;
	} else {
		return (!required || (!(this.path in data) && item && item.get(this.path))) ? true : false;
	}
};

/**
 * Formats the field value
 */
// qrcode.prototype.format = function (item) {
// 	return this.labels[item.get(this.path)] || '';
// };

/**
 * Get the value from a data object; may be simple or a pair of fields
 */
goodpropertymarix.prototype.getInputFromData = function (data) {
	// this.getValueFromData throws an error if we pass name: null
	if (data[this.path] === null || data[this.path] === undefined || data[this.path] === '') {
		return null;
	}
	else if (typeof data[this.path] == "object"){
		return data[this.path];
	}
	return JSON.parse(data[this.path]);

};

goodpropertymarix.prototype.updateItem = function (item, data, callback) {
	var path = this.path;
	var value = this.getInputFromData(data);
	if (typeof value === 'object') {
		
		item.set(path, value);
	}
	process.nextTick(callback);
};

/* Export Field Type */
module.exports = goodpropertymarix;
